/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.multiplexer;

import java.util.Map;

import org.objectweb.dream.Pull;
import org.objectweb.dream.Push;

/**
 * This class defines a periodic attachment.
 */
public class PeriodicAttachment extends Attachment
{
  protected long             startingDate;
  protected long             pullingFrequencyInMillis;
  protected long             endDate;                               // 0 means
                                                                    // no end
  protected long             nextDeadline      = 0;

  /**
   * The name of the key that must be used to specify the starting date of this
   * attachment in the <code>parameters</code> passed to the
   * {@link org.objectweb.dream.multiplexer.PullPushMultiplexer#attach(String[], Map[], String[], Map[], Map) }
   */
  public static final String STARTING_DATE     = "startingDate";

  /**
   * The name of the key that must be used to specify the end date of this
   * attachment in the <code>parameters</code> passed to the
   * {@link org.objectweb.dream.multiplexer.PullPushMultiplexer#attach(String[], Map[], String[], Map[], Map) }
   */
  public static final String END_DATE          = "endDate";

  /**
   * The name of the key that must be used to specify the pulling frequency of
   * this attachment in the <code>parameters</code> passed to the
   * {@link org.objectweb.dream.multiplexer.PullPushMultiplexer#attach(String[], Map[], String[], Map[], Map) }
   */
  public static final String PULLING_FREQUENCY = "pullingFrequency";

  /**
   * The name of the key that must be used in the context of the
   * <code>pull</code> method to specify the time at which the pulling has
   * been scheduled by the multiplexer.
   */
  public static final String TIMESTAMP         = "timestamp";

  /**
   * Creates a new <code>Attachment</code> object
   * 
   * @param id the id of the attachment
   * @param inputNames the names of the attached inputs.
   * @param inputs the attached inputs.
   * @param inputContexts the contexts to be passed when pulling the inputs.
   * @param outputNames the names of the outputs to wich intputs are attached.
   * @param outputs the outputs to wich intputs are attached.
   * @param outputContexts the contexts to be passed when pushing the outputs.
   * @param startingDate attachment start date.
   * @param pullingFrequencyInMillis time between two pulling
   * @param endDate attachment end date
   */
  public PeriodicAttachment(int id, String[] inputNames, Pull[] inputs,
      Map[] inputContexts, String[] outputNames, Push[] outputs,
      Map[] outputContexts, long startingDate, long pullingFrequencyInMillis,
      long endDate)
  {
    super(id, inputNames, inputs, inputContexts, outputNames, outputs,
        outputContexts);
    this.startingDate = startingDate;
    this.pullingFrequencyInMillis = pullingFrequencyInMillis;
    this.endDate = endDate;
    this.nextDeadline = startingDate;
  }

  /**
   * Creates a new <code>Attachment</code> object without an end date. It
   * means that pulling never ends.
   * 
   * @param id the id of the attachment
   * @param inputNames the names of the attached inputs.
   * @param inputs the attached inputs.
   * @param inputContexts the contexts to be passed when pulling the inputs.
   * @param outputNames the names of the outputs to wich intputs are attached.
   * @param outputs the outputs to wich intputs are attached.
   * @param outputContexts the contexts to be passed when pushing the outputs.
   * @param startingDate attachment start date.
   * @param pullingFrequencyInMillis time between two pulling
   */
  public PeriodicAttachment(int id, String[] inputNames, Pull[] inputs,
      Map[] inputContexts, String[] outputNames, Push[] outputs,
      Map[] outputContexts, long startingDate, long pullingFrequencyInMillis)
  {
    this(id, inputNames, inputs, inputContexts, outputNames, outputs,
        outputContexts, startingDate, pullingFrequencyInMillis, 0);
  }

  /**
   * Returns the nextDeadline value.
   * 
   * @return the nextDeadline.
   */
  public long getNextDeadline()
  {
    return nextDeadline;
  }

  /**
   * Sets the nextDeadline value.
   * 
   * @param nextDeadline the nextDeadline to set.
   */
  public void setNextDeadline(long nextDeadline)
  {
    this.nextDeadline = nextDeadline;
  }

  /**
   * Returns the startingDate value.
   * 
   * @return the starting date.
   */
  public long getStartingDate()
  {
    return startingDate;
  }

  /**
   * Returns the endDate value.
   * 
   * @return the end date.
   */
  public long getEndDate()
  {
    return endDate;
  }

  /**
   * Sets the endDate value.
   * 
   * @param endDate the endDate to set.
   */
  public void setEndDate(long endDate)
  {
    this.endDate = endDate;
  }

  /**
   * Returns the samplingFrequencyInMillis value.
   * 
   * @return the samplingFrequencyInMillis.
   */
  public long pullingFrequencyInMillis()
  {
    return pullingFrequencyInMillis;
  }

  /**
   * Returns a clone of this attachment.
   * 
   * @return a clone of this attachment.
   */
  public Object clone()
  {
    return super.clone();
  }

}