/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.multiplexer;

import java.util.ArrayList;
import java.util.Map;

import org.objectweb.fractal.api.NoSuchInterfaceException;

/**
 * This interface defines a pull-push multiplexer. This multiplexer has two
 * <i>collection </i> client interfaces:
 * <ul>
 * <li>a Pull client collection interface for incoming messages (input).</li>
 * <li>a Push client collection interface for outgoing messages (output).</li>
 * </ul>
 * Inputs can be attached (detached) to (from) outputs using the attach (detach)
 * method.
 */
public interface PullPushMultiplexer
{

  /**
   * The commonly used name to refer to this interface.
   */
  String ITF_NAME = "multiplexer";

  /**
   * Attaches a set of inputs to a set of outputs.
   * 
   * @param inputNames the names of the inputs to be attached. These inputs must
   *          have been previously bound (with the same name) using the
   *          <code>BindingController</code> interface.
   * @param inputContexts the contexts to be passed when pulling the inputs.
   * @param outputNames the names of the outputs to wich intputs must be
   *          attached. These outputs must have been previously bound (with the
   *          same name) using the <code>BindingController</code> interface.
   * @param outputContexts the contexts to be passed when pushing the outputs.
   * @param parameters the attachement parameters.
   * @return an <code>Attachment</code> object
   * @throws NoSuchInterfaceException if inputNames or outputNames contains the
   *           name of an interface to which the component has not been bound.
   */
  Attachment attach(String[] inputNames, Map[] inputContexts,
      String[] outputNames, Map[] outputContexts, Map parameters)
      throws NoSuchInterfaceException;

  /**
   * Detaches a set of inputs from a set of outputs.
   * 
   * @param attachment an attachment returned by <code>attach</code>
   */
  void detach(Attachment attachment);

  /**
   * Updates an attachement. New parameters are given in an
   * <code>Attachment</code>.
   * 
   * @param attachment an attachment returned by <code>attach</code>.
   */
  void update(Attachment attachment);

  /**
   * Returns the attachments that have been registered.
   * 
   * @return the attachments that have been registered.
   */
  ArrayList getAttachments();
}