/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s):
 */

package org.objectweb.dream.protocol.causality;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.util.EmptyStringArray;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Basic Implementation of a matrix clock. Stamps used by this component are
 * integer arrays. This implementation is not efficient if the number of process
 * involved in the causality order is important.
 */
public class BasicMatrixClockImpl extends AbstractComponent
    implements
      MatrixClock,
      MatrixClockAttributeController
{

  private int   matrix[][] = null;
  private short size       = -1;

  // ---------------------------------------------------------------------------
  // Implementation of MatrixClock interface
  // ---------------------------------------------------------------------------

  /**
   * @see MatrixClock#testRecvMatrix(Object, short, short)
   */
  public int testRecvMatrix(Object stamp, short from, short to)
  {
    int[][] stampTab = (int[][]) stamp;
    if (matrix[from][to] + 1 < stampTab[from][to])
    {
      // There is other messages from the same node to deliver before this one.
      return WAIT_TO_DELIVER;
    }
    else if (matrix[from][to] + 1 == stampTab[from][to])
    {
      // Verify that all messages to be delivred to this node and known by this
      // message have already been delivred.
      for (int k = 0; k < matrix.length; k++)
      {
        if ((k != from) && (stampTab[k][to] > matrix[k][to]))
        {
          return WAIT_TO_DELIVER;
        }
      }
    }
    else
    {
      return ALREADY_DELIVERED;
    }
    // the message is ready to be delivred, so updates the matrix clock
    for (int i = 0; i < stampTab.length; i++)
    {
      for (int j = 0; j < stampTab.length; j++)
      {
        if (stampTab[i][j] > matrix[i][j])
        {
          matrix[i][j] = stampTab[i][j];
        }
      }
    }
    return DELIVER;
  }

  /**
   * @see MatrixClock#getStamp(short, short)
   */
  public Object getStamp(short from, short to)
  {
    // increment matrix clock
    matrix[from][to]++;
    // copy the matrix and set the copy as stamp.
    int[][] stamp = new int[matrix.length][matrix.length];
    for (int i = 0; i < stamp.length; i++)
    {
      System.arraycopy(matrix[i], 0, stamp[i], 0, matrix.length);
    }
    return stamp;
  }

  // ---------------------------------------------------------------------------
  // Implementation of MatrixClockAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.protocol.causality.MatrixClockAttributeController#getNumberOfProcess()
   */
  public short getNumberOfProcess()
  {
    return size;
  }

  /**
   * @see org.objectweb.dream.protocol.causality.MatrixClockAttributeController#setNumberOfProcess(short)
   */
  public void setNumberOfProcess(short numberOfProcess)
  {
    if (matrix != null && numberOfProcess != size)
    {
      throw new IllegalStateException(
          "Matrix clock is already initialized, cannot change its size");
    }
    size = numberOfProcess;
    matrix = new int[size][size];
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return EmptyStringArray.EMPTY_STRING_ARRAY;
  }

  // ---------------------------------------------------------------------------
  // Overriding of LifeCycleController methods
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    if (matrix == null)
    {
      throw new IllegalLifeCycleException("Matrix size has not been set");
    }
    super.startFc();
  }
}