/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s):
 */

package org.objectweb.dream.protocol.causality;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.util.EmptyStringArray;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Matric clock implementation using
 * {@link org.objectweb.dream.protocol.causality.Update }as stamp. More
 * efficient for large matrix.
 */
public class UpdateMatrixClockImpl extends AbstractComponent
    implements
      MatrixClock,
      MatrixClockAttributeController
{

  private final class MatrixElt
  {
    /** Element value. */
    int stamp;
    /** Source node of last modification. */
    int node;
    /** State value when last modified. */
    int status;
  }

  private int       status[];
  private short     size = -1;
  private MatrixElt matrix[][];

  /**
   * @see MatrixClock#testRecvMatrix(Object, short, short)
   */
  public int testRecvMatrix(Object stamp, short from, short to)
  {

    Update ptr = (Update) stamp;
    if ((matrix[from][to].stamp + 1) < ptr.getStamp())
    {
      // There is other messages from the same node to deliver before this one.
      return WAIT_TO_DELIVER;
    }
    else if ((matrix[from][to].stamp + 1) == ptr.getStamp())
    {
      // Verify that all messages to be delivred to this node and known by this
      // message have already been delivred.
      ptr = ptr.getNext();
      while (ptr != null)
      {
        if ((ptr.getColumn() == to)
            && ptr.getStamp() > matrix[ptr.getLine()][ptr.getColumn()].stamp)
        {
          break;
        }
        ptr = ptr.getNext();
      }
    }
    else
    {
      return ALREADY_DELIVERED;
    }

    if (ptr != null)
    {
      return WAIT_TO_DELIVER;
    }
    else
    {
      // the message is ready to be delivred, so updates the matrix clock
      ptr = (Update) stamp;
      do
      {
        MatrixElt elt = matrix[ptr.getLine()][ptr.getColumn()];
        if (elt.stamp < ptr.getStamp())
        {
          elt.stamp = ptr.getStamp();
          elt.node = from;
          elt.status = status[to];
        }
        Update nextUpdate = ptr.getNext();
        Update.unalloc(ptr);
        ptr = nextUpdate;
      }
      while (ptr != null);
      return DELIVER;
    }
  }

  /**
   * @see MatrixClock#getStamp(short, short)
   */
  public Object getStamp(short from, short to)
  {
    MatrixElt elt = matrix[from][to];
    elt.stamp++;
    elt.status = status[from];
    elt.node = to;

    Update update = Update.alloc(from, to, elt.stamp, null);

    if (to != from)
    {
      // if the message is remote there is need of matrix clock update.
      for (short i = 0; i < matrix.length; i++)
      {
        for (short j = 0; j < matrix[i].length; j++)
        {
          MatrixElt elt1 = matrix[i][j];
          if ((elt1.status > status[to]) && (elt1.node != to)
              && ((i != from) || (j != to)))
          {
            Update.alloc(i, j, elt1.stamp, update);
          }
        }
      }
      status[to] = status[from];
      status[from]++;
    }

    return update;
  }

  // ---------------------------------------------------------------------------
  // Implementation of MatrixClockAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.protocol.causality.MatrixClockAttributeController#getNumberOfProcess()
   */
  public short getNumberOfProcess()
  {
    return size;
  }

  /**
   * @see org.objectweb.dream.protocol.causality.MatrixClockAttributeController#setNumberOfProcess(short)
   */
  public void setNumberOfProcess(short numberOfProcess)
  {
    if (matrix != null && numberOfProcess != size)
    {
      throw new IllegalStateException(
          "Matrix clock is already initialized, cannot change its size");
    }
    size = numberOfProcess;
    matrix = new MatrixElt[size][size];
    status = new int[size];
    for (short i = 0; i < matrix.length; i++)
    {
      for (short j = 0; j < matrix[i].length; j++)
      {
        matrix[i][j] = new MatrixElt();
      }
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return EmptyStringArray.EMPTY_STRING_ARRAY;
  }

  // ---------------------------------------------------------------------------
  // Overriding of LifeCycleController methods
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    if (matrix == null)
    {
      throw new IllegalLifeCycleException("Matrix size has not been set");
    }
    super.startFc();
  }
}