/**
 * DREAM
 * Copyright (C) 2003 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: matthieu.leclercq@inrialpes.fr
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import java.util.LinkedList;
import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Pull;
import org.objectweb.dream.Push;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.util.EmptyStringArray;

/**
 * High performance implementation of a Push/Pull queue. This queue is not
 * thread safe and its <i>pull </i> interface is non blocking (return
 * <code>null</code> if the buffer is empty).
 */
public class NotSynchronizedPushPullQueueImpl extends AbstractComponent
    implements
      Push,
      Pull,
      PushQueueAttributeController
{

  /**
   * The LinkedList containing waiting message
   */
  protected LinkedList waitingList = new LinkedList();

  /**
   * @see Push#push(org.objectweb.dream.message.Message, java.util.Map)
   */
  public void push(Message message, Map context)
  {
    waitingList.add(message);
  }

  /**
   * @see Pull#pull(java.util.Map)
   */
  public Message pull(Map context)
  {
    if (waitingList.isEmpty())
    {
      return null;
    }
    else
    {
      return (Message) waitingList.removeFirst();
    }
  }

  /**
   * @see org.objectweb.dream.queue.PushQueueAttributeController#getOverflowPolicy()
   */
  public String getOverflowPolicy()
  {
    return EXCEPTION_OVERFLOW_POLICY;
  }

  /**
   * @see org.objectweb.dream.queue.PushQueueAttributeController#setOverflowPolicy(java.lang.String)
   */
  public void setOverflowPolicy(String policy)
  {
    // do nothing, no overflow in this queue
  }

  /**
   * @see org.objectweb.dream.queue.QueueAttributeController#getCurrentSize()
   */
  public int getCurrentSize()
  {
    return waitingList.size();
  }

  /**
   * @see org.objectweb.dream.queue.QueueAttributeController#getMaxCapacity()
   */
  public int getMaxCapacity()
  {
    return 0;
  }

  /**
   * @see org.objectweb.dream.queue.QueueAttributeController#setMaxCapacity(int)
   */
  public void setMaxCapacity(int maxCapacity)
  {
    // do nothing no max capacity
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return EmptyStringArray.EMPTY_STRING_ARRAY;
  }

}